////////////////////////////////////////////////////////////////////////////////
//      _    _  ____            ____    // LoadSave.pas: unit LoadSave        //
//     / \  | |/ ___| ___ _ __ |___ \   //------------------------------------//
//    / _ \ | | |  _ / _ \ '_ \  __) |  // Routinen zum Laden und Speichern   //
//   / ___ \| | |_| |  __/ | | |/ __/   // der Einstellungen und der Bilder-  //
//  /_/   \_\_|\____|\___|_| |_|_____|  // liste in bzw. aus Pseudo-XML-      //
//                                      // Dateien                            //
////////////////////////////////////////////////////////////////////////////////
unit LoadSave;
interface

procedure SaveProject(const FileName:string);  // Projekt in Datei speichern
procedure LoadProject(const FileName:string);  // Projekt aus Datei laden


implementation
uses SysUtils, Forms, ComCtrls,
     MLParse, Datacore, Main, Acquire;


///// BoolStr: "konvertiert" ein Boolean-Wert in einen angegebenen String
function BoolStr(b:boolean; const s0,s1:string):string;
begin
  if b then Result:=s1 else Result:=s0;
end;


//// StrBool: interpretiert einen String-Wert als Boolean
function StrBool(s:string):boolean;
begin
  s:=LowerCase(s);
  Result:=not((s='') OR (s='0') OR (s='no') OR (s='non') OR (s='nein') OR
              (s='off') OR (s='disabled'));
end;


//// StrSpin: konvertiert einen String in einen Integerwert und stellt ihn
//// gleich in ein TUpDown-Control ein
procedure StrSpin(Spin:TUpDown; const s:string);
begin try Spin.Position:=StrToInt(s); except end; end;


///// WARNUNG: Es folgen fast 300 Zeilen wirrer, aber stupider, monotoner Code!

////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////// SaveProject
////////////////////////////////////////////////////////////////////////////////

procedure SaveProject(const FileName:string);
var Root:TMLTag;
    Index,Category:integer;
begin with MainForm do begin
  // Root-Tag erzeugen
  Root:=TMLTag.Create('AlGen2Project');
  if MyVersion<>'' then Root.AddAttribute('Version',MyVersion);
  // Konfiguration
  with Root.CreateSubItem('Configuration') do begin
    if CVerbose.Checked then AddAttribute('Verbose','-v');
    CreateSubItem('DestDir').AddAttribute('Value',EDestDir.Text);
    CreateSubItem('AlbumTitle').AddAttribute('Value',ETitle.Text);
    // Unterabschnitt: Preprocessing
    with CreateSubItem('Preprocessing') do begin
      AddAttribute('MedianFilter',BoolStr(CMedianFilter.Checked,'off','on'));
      AddAttribute('Equalize',BoolStr(CEnableEqualize.Checked,'no','yes'));
      AddAttribute('EqualizeStrength',IntToStr(CEqualizeStrength.Position));
      AddAttribute('EqualizeMode',BoolStr(CEqualizeRGB.Checked,'Gray','RGB'));
    end;
    // Unterabschnitt: Vorschaubilder
    with CreateSubItem('PreviewImages') do begin
      AddAttribute('Width',IntToStr(CPSizeX.Position));
      AddAttribute('Height',IntToStr(CPSizeY.Position));
      AddAttribute('Aspect',BoolStr(CPQuadratic.Checked,'free','quadratic'));
      AddAttribute('Quality',IntToStr(CPQuality.Position));
      AddAttribute('Progressive',BoolStr(CPProgressive.Checked,'no','yes'));
    end;
    // Unterabschnitt: Thumbnails
    with CreateSubItem('ThumbnailImages') do begin
      AddAttribute('Width',IntToStr(CTSizeX.Position));
      AddAttribute('Height',IntToStr(CTSizeY.Position));
      AddAttribute('Aspect',BoolStr(CTQuadratic.Checked,'free','quadratic'));
      AddAttribute('Quality',IntToStr(CTQuality.Position));
      AddAttribute('Progressive',BoolStr(CTProgressive.Checked,'no','yes'));
      if CEnableFrame.Checked then
        AddAttribute('FrameSize',IntToStr(CFrameSize.Position));
    end;
    // Unterabschnitt: Ausgabe
    with CreateSubItem('Output') do begin
      case COutputHTML.ItemIndex of
        0:AddAttribute('Mode','NoHTML');
        1:AddAttribute('Mode','ThumbsOnly');
        2:AddAttribute('Mode','IndexOnly');
        3:AddAttribute('Mode','WithGallery');
        4:AddAttribute('Mode','WithDetails');
      end;
      AddAttribute('ThumbsPerLine',IntToStr(CThumbsPerLine.Position));
      if CUncatIgnore.Checked
        then AddAttribute('UncategorizedImages','Ignore')
        else AddAttribute('UncategorizedImages',BoolStr(CUncatAtEnd.Checked,'AtStart','AtEnd'));
      if Crmrf.Checked then AddAttribute('rm','-rf');
      AddAttribute('FullWidth',BoolStr(CFullWidth.Checked,'no','yes'));
    end;
    // Unterabschnitt: Dateinamen
    with CreateSubItem('FileNames') do begin
      with CreateSubItem('Preview') do begin
        AddAttribute('Template',EfnPreview.Text);
        AddAttribute('UseOwnDirectory',BoolStr(CdirPreview.Checked,'no','yes'));
        AddAttribute('OwnDirectory',EdirPreview.Text);
      end;
      with CreateSubItem('Thumbnails') do begin
        AddAttribute('Template',EfnThumbnails.Text);
        AddAttribute('UseOwnDirectory',BoolStr(CdirThumbnails.Checked,'no','yes'));
        AddAttribute('OwnDirectory',EdirThumbnails.Text);
      end;
      with CreateSubItem('HTMLDocs') do begin
        AddAttribute('Index',EfnIndex.Text);
        AddAttribute('Gallery',EfnHTML.Text);
        AddAttribute('Details',EfnInfo.Text);
      end;
      AddAttribute('Digits',IntToStr(CDigits.Position));
    end;
  end;
  // Bilderliste
  with Root.CreateSubItem('ImageList') do begin
    AddAttribute('Count',IntToStr(Data.ImageCount));
    // Kategorienweise <Category>-Tags erzeugen
    for Category:=0 to Data.CategoryCount do
      with CreateSubItem('Category') do begin
        if Category>0 then
          AddAttribute('Name',Data.Categories[Category]);
        // Bilder einer Kategorie in <Image>-Tags speichern
        for Index:=0 to (Data.ImageCount-1) do
          if Data[Index]^.Category=Category then
            with CreateSubItem('Image'),Data[Index]^ do begin
              AddAttribute('Source',PathName);
              AddAttribute('Active',BoolStr(Active,'no','yes'));
              case Rotation of
                rot0:AddAttribute('Rotate','0');
                rot90CW:AddAttribute('Rotate','90CW');
                rot90CCW:AddAttribute('Rotate','90CCW');
                rot180:AddAttribute('Rotate','180');
              end;
              AddAttribute('Caption',Comment);
            end;
      end;
  end;
  // Struktur ist erstellt, es kann gespeichert werden
  Log('');
  if Root.DumpToFile(FileName) then
    Log('*Projekt abgespeichert in '+FileName)
  else begin
    Log('_Fehler beim Speichern des Projektes in '+FileName);
    PageThing.ActivePage:=PLog;
  end;
  Log('');
  Root.Free;
end; end;


////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////// LoadProject
////////////////////////////////////////////////////////////////////////////////

procedure LoadProject(const FileName:string);
var PFile,L1,L2,L3:TMLTag; Temp:string;
    Count,Total,Cat,Index,Success:integer;
begin with MainForm do begin
  // Vorbereitungen
  SelectOp(nil);
  Log('');
  Log('/Lese Projektdatei '+FileName);
  PFile:=TMLTag.Create('');
  try  // ein riesiges try...except, um fr alle Eventualitten gerstet zu sein
    // Datei lesen, alles weitere im Root-Tag suchen
    PFile.ParseFromFile(FileName);
    with PFile.GetSubItemByName('AlGen2Project') do begin
      // Konfiguration
      L1:=GetSubItemByName('Configuration');
      if Assigned(L1) then with L1 do begin
        CVerbose.Checked:=StrBool(GetAttributeByName('Verbose'));
        L2:=GetSubItemByName('DestDir');
        if Assigned(L2) then EDestDir.Text:=L2.GetAttributeByName('Value');
        L2:=GetSubItemByName('AlbumTitle');
        if Assigned(L2) then ETitle.Text:=L2.GetAttributeByName('Value');
        // Unterabschnitt: Preprocessing
        L2:=GetSubItemByName('Preprocessing');
        if Assigned(L2) then with L2 do begin
          CMedianFilter.Checked:=StrBool(GetAttributeByName('MedianFilter'));
          CEnableEqualize.Checked:=StrBool(GetAttributeByName('Equalize'));
          StrSpin(CEqualizeStrength,GetAttributeByName('EqualizeStrength'));
          Temp:=UpperCase(GetAttributeByName('EqualizeMode'));
          if Temp='RGB' then CEqualizeRGB.Checked:=true
                        else CEqualizeGray.Checked:=true;
        end;
        // Unterabschnitt: Vorschaubilder
        L2:=GetSubItemByName('PreviewImages');
        if Assigned(L2) then with L2 do begin
          StrSpin(CPSizeX,GetAttributeByName('Width'));
          StrSpin(CPSizeY,GetAttributeByName('Height'));
          CPQuadratic.Checked:=(LowerCase(copy(GetAttributeByName('Aspect'),1,4))='quad');
          StrSpin(CPQuality,GetAttributeByName('Quality'));
          CPProgressive.Checked:=StrBool(GetAttributeByName('Progressive'));
        end;
        // Unterabschnitt: Thumbnails
        L2:=GetSubItemByName('ThumbnailImages');
        if Assigned(L2) then with L2 do begin
          StrSpin(CTSizeX,GetAttributeByName('Width'));
          StrSpin(CTSizeY,GetAttributeByName('Height'));
          CTQuadratic.Checked:=(LowerCase(copy(GetAttributeByName('Aspect'),1,4))='quad');
          StrSpin(CTQuality,GetAttributeByName('Quality'));
          CTProgressive.Checked:=StrBool(GetAttributeByName('Progressive'));
          Temp:=GetAttributeByName('FrameSize');
          CEnableFrame.Checked:=(Temp<>'');
          StrSpin(CFrameSize,Temp);
        end;
        // Unterabschnitt: Ausgabe
        L2:=GetSubItemByName('Output');
        if Assigned(L2) then with L2 do begin
          Temp:=LowerCase(GetAttributeByName('Mode'));
          if pos('thumb',Temp)>0 then COutputHTML.ItemIndex:=1 else
          if pos('index',Temp)>0 then COutputHTML.ItemIndex:=2 else
          if pos('gal',Temp)>0 then COutputHTML.ItemIndex:=3 else
          if (pos('detail',Temp)>0) OR (pos('info',Temp)>0) then COutputHTML.ItemIndex:=4
          else COutputHTML.ItemIndex:=0;
          StrSpin(CThumbsPerLine,GetAttributeByName('ThumbsPerLine'));
          Temp:=LowerCase(GetAttributeByName('UncategorizedImages'));
          if pos('ignore',Temp)>0 then CUncatIgnore.Checked:=true;
          if pos('start',Temp)>0 then CUncatAtStart.Checked:=true;
          if pos('end',Temp)>0 then CUncatAtEnd.Checked:=true;
          Crmrf.Checked:=StrBool(GetAttributeByName('rm'));
          CFullWidth.Checked:=StrBool(GetAttributeByName('FullWidth'));
        end;
        // Unterabschnitt: Dateinamen
        L2:=GetSubItemByName('FileNames');
        if Assigned(L2) then with L2 do begin
          StrSpin(CDigits,GetAttributeByName('Digits'));
          L3:=GetSubItemByName('Preview');
          if Assigned(L3) then with L3 do begin
            Temp:=GetAttributeByName('Template');
            if Temp<>'' then EfnPreview.Text:=Temp;
            CdirPreview.Checked:=StrBool(GetAttributeByName('UseOwnDirectory'));
            Temp:=GetAttributeByName('OwnDirectory');
            if Temp<>'' then EdirPreview.Text:=Temp;
          end;
          L3:=GetSubItemByName('Thumbnails');
          if Assigned(L3) then with L3 do begin
            Temp:=GetAttributeByName('Template');
            if Temp<>'' then EfnThumbnails.Text:=Temp;
            CdirThumbnails.Checked:=StrBool(GetAttributeByName('UseOwnDirectory'));
            Temp:=GetAttributeByName('OwnDirectory');
            if Temp<>'' then EdirThumbnails.Text:=Temp;
          end;
          L3:=GetSubItemByName('HTMLDocs');
          if Assigned(L3) then with L3 do begin
            Temp:=GetAttributeByName('Index');
            if Temp<>'' then EfnIndex.Text:=Temp;
            Temp:=GetAttributeByName('Gallery');
            if Temp<>'' then EfnHTML.Text:=Temp;
            Temp:=GetAttributeByName('Details');
            if Temp<>'' then EfnInfo.Text:=Temp;
          end;
        end;
      end else
        Log('-Keine Konfigurationsdaten');
      // Bilderliste
      Count:=0; Success:=0;
      L1:=GetSubItemByName('ImageList');
      if Assigned(L1) then with L1 do begin
        // Bilderliste liegt vor -> Liste resetten
        Data.Free;
        Data:=TImageManager.Create;
        // Gesamtbilderzahl lesen, falls vorhanden
        try
          Total:=StrToInt(GetAttributeByName('Count'))
        except
          Total:=0;
        end;
        // kategorienweise einlesen
        for Cat:=0 to (SubItemCount-1) do with SubItems[Cat] do begin
          if Cat>0 then Data.AddCategory(GetAttributeByName('Name'));
          for Index:=0 to (SubItemCount-1) do with SubItems[Index] do if Name='Image' then begin
            // Bildname holen
            Temp:=GetAttributeByName('Source');
            if Temp<>'' then begin
              inc(Count);
              // Bild einlesen
              if AcquireImage(Temp) then begin
                Log('*['+IntToStr(Count)+'/'+IntToStr(Total)+'] Bild hinzugefgt: '+Temp);
                inc(Success);
                with Data[Data.CountImages-1]^ do begin
                  Category:=Cat;
                  Active:=StrBool(GetAttributeByName('Active'));
                  Temp:=UpperCase(GetAttributeByName('Rotate'));
                  if (Temp='90CW') OR (Temp='90') then Rotation:=rot90CW;
                  if (Temp='90CCW') OR (Temp='270') then Rotation:=rot90CCW;
                  if Temp='180' then Rotation:=rot180;
                  Comment:=GetAttributeByname('Caption');
                end;
              end else
                // konnte Bild nicht einlesen
                Log('-['+IntToStr(Count+1)+'/'+IntToStr(Total)+'] Fehler beim Hinzufgen: '+Temp);
            end;
            // Nutzerinteraktion zulassen
            Application.ProcessMessages;
            if AbortOp then break;
          end;
        end;
        // Ende
        if AbortOp then Log('-Vorgang vom Benutzer abgebrochen');
        Log('+'+IntToStr(Success)+' von '+IntToStr(Total)+
            ' Dateien erfolgreich hinzugefgt');
      end else
        Log('-Keine Bilderlistendaten');
    end;
    // Fertig mit Parsen
  except
    Log('-Fehler beim Lesen der Projektdatei');
  end;
  // Aufrumen
  PFile.Free;
  Log('');
  Data.Rehash;
  Data.CategoriesToStrings(-1,CCategory.Items);
  CCategory.ItemIndex:=0;
  CCategoryChange(nil);
  SelectOp(PLog);
end; end;

end.
